import React, { Component } from 'react';
import ReactDOM from 'react-dom';
const Context = React.createContext();;

export default class Parallax extends Component {
  constructor(props) {
    super(props);

    this.layers = [];
  }

  addLayer = (layer) => {
    this.layers.push(layer);
  }

  removeLayer = (layer) => {
    this.layers = this.layers.filter(ly => ly !== layer);
  }

  getContext = () => {
    return {
      addLayer: this.addLayer,
      removeLayer: this.removeLayer,
    }
  }

  componentDidMount() {
    this.setup();
  }

  componentWillUnmount() {
    window.removeEventListener('scroll', this.handleScroll);
  }

  setup() {
    window.addEventListener('scroll', this.handleScroll);
    window.dispatchEvent(new Event('scroll'));
  }

  handleScroll = () => {
    window.requestAnimationFrame(this.checkParallaxItems)
  }

  checkParallaxItems = () => {
    this.layers.forEach(layer => {
      const item = ReactDOM.findDOMNode(layer);
      const position = item.getBoundingClientRect();
      if (position.top < window.innerHeight && position.bottom >= 0) {
        this.draw(item, position, layer.settings);
      }
    });
  }

  draw = (item, position, settings = { type: 'backgroundY', speed: 0.3 }) => {
    const speed = settings.speed;
    let newValue = -Math.floor(speed * position.top);

    const min = settings.min || (speed < 0 ? 0 : newValue);
    const max = settings.max || (speed > 0 ? 0 : newValue);

    if (newValue < min) {
      newValue = min;
    } else if (newValue > max) {
      newValue = max;
    }

    const updatedValues = {};
    if (Array.isArray(settings.type)) {
      settings.type.forEach(t => {
        const { key, value } = this.getChangedValue(t, newValue);
        if (!updatedValues[key]) {
          updatedValues[key] = [];
        }
        updatedValues[key].push(value);
      })
    } else {
      const { key, value } = this.getChangedValue(settings.type, newValue);
      updatedValues[key] = [value];
    }

    // eslint-disable-next-line
    for (let key in updatedValues) {
      item.style[key] = updatedValues[key].join(' ');
    }
  }

  getChangedValue = (type, value) => {
    switch (type) {
      case 'translateY':
        return {
          key: 'transform',
          value: `translateY(${value}px)`
        };

      case 'translateX':
        return {
          key: 'transform',
          value: `translateX(${value}px)`
        }

      case 'scale':
        return {
          key: 'transform',
          value: `scale(${value / 100})`
        }

      case 'rotate':
        return {
          key: 'transform',
          value: `rotate(${value}deg)`
        }

      case 'backgroundPositionX':
        return {
          key: 'backgroundPosition',
          value: `${value}px center`
        };

      default:
        return {
          key: 'backgroundPosition',
          value: `center ${value}px`
        };
    }
  }

  render() {
    return (
      <Context.Provider value={this.getContext()}>
        {this.props.children}
      </Context.Provider>
    )
  }
}

export class Layer extends Component {
  constructor(props) {
    super(props);
    this.settings = props.settings;
  }

  componentDidMount() {
    this.context.addLayer(this);
  }

  componentWillUnmount() {
    this.context.removeLayer(this);
  }

  render() {
    const props = { ...this.props, settings: null };
    return <div {...props}>
      {this.props.children}
    </div>
  }
}

Layer.contextType = Context;